//copy of https://github.com/fuze/meetings/blob/fcc0317d865dd14bb3625f04ab3669fd77ad4e39/Clients/html5/webroot/js/fuze/util/utils.js
var CClass = {
    create : function(fn) {
        var parent = (typeof this === "function") ? this : function(){};
        var constr = function() {
            parent.apply(this, arguments);
            fn.apply(this, arguments);
        };
        constr.extend = this.extend || this.create;
        return constr;
    }
};

var Timer = function(milis) {

    var intervalId = null;
    var started    = false;

    this.start = function(fn) {
        if (!started) {
            intervalId = setInterval(fn, milis);
            started = true;
        }
    };

    this.stop = function() {
        if (started) {
            clearInterval(intervalId);
            started = false;
        }
    };
};

Timer.once = function(cb, timeout) {
    var id = setTimeout(function() {
        cb();
    }, timeout);
    return id;
};
Timer.clearOnce = function(id) {
    clearTimeout(id);
};


var StringBuilder = function() {

    var chunks = [];
    var len = 0;

    this.append = function(str) {
        if (str) {
            chunks.push(str);
            len += str.length;
        }
        return this; //allows chaining
    };

    this.length = function() {
        return len;
    };

    this.toString = function() {
        return chunks.join('');
    };
};

var MathUtil = {};

MathUtil.fitRect = function(boundsWW, boundsHH, targetToFitWW, targetToFitHH) {
    targetToFitWW = (targetToFitWW === 0) ? 1: targetToFitWW; //prevent divide by 0
    targetToFitHH = (targetToFitHH === 0) ? 1: targetToFitHH; //prevent divide by 0

    var wwRatio = boundsWW/targetToFitWW;
    var hhRatio = boundsHH/targetToFitHH;
    var scale = (wwRatio < hhRatio) ? wwRatio : hhRatio;
    var ww = Math.round(targetToFitWW * scale);
    var hh = Math.round(targetToFitHH * scale);
    return { width: ww, height: hh };
};

MathUtil.hexToDec = function(hex) {
    var intDec = parseInt(hex, 16);
    return intDec;
};

var JSUtil = {
    isNumber: function(arg) {
        return typeof arg === "number";
    },
    isNullOrUndefined: function(arg) {
        return (arg === null || arg === undefined);
    },
    isNull: function(arg) {
        return (arg === null);
    },
    isUndefined: function(arg) {
        return (arg === undefined);
    },
    isFunction: function(arg) {
        return (typeof arg === "function");
    },
    isObject: function(arg) {
        if (JSUtil.isNull(arg)) {
            return false;
        }
        return (typeof arg === "object");
    },
    isString: function(arg) {
        return (typeof arg === "string");
    },

    countOwnProperties: function(obj) {
        var count = 0;
        for (var k in obj) {
            if (obj.hasOwnProperty(k)) {
                count++;
            }
        }
        return count;
    }
};

var ValidationUtil = {
    strIsEmpty: function(val) {
        return (!val || !val.length);
    },
    isPosInt: function(val) {
        var n = ~~Number(val);
        return String(n) === String(val) && n >= 0;
    },
    isPhoneNumber: function(val) {
        if (val) {
            var regexp = /^\+?[0-9\s(\-)]+$/;
            return !!val.match(regexp);
        } else {
            return false;
        }
    },
    isInternationalPhoneNumber: function(val) {
        if (val) {
            var regexp = /^(\+|00)[0-9\s(\-)]+$/;
            return !!val.match(regexp);
        } else {
            return false;
        }
    },
    isValidEmail: function(val) {
        if (val) {
            var regexp = /[^@]+@[^@]+\.[a-zA-Z]{2,6}/;
            return !!val.match(regexp);
        } else {
            return false;
        }
    }
};

var DateUtil = {
    formatStats: function(date) {
        if (!date) {
            return undefined;
        }
        var hour   = StringUtil.zeroLPad(date.getHours(), 2);
        var min    = StringUtil.zeroLPad(date.getMinutes(), 2);
        var secs   = StringUtil.zeroLPad(date.getSeconds(), 2);

        var year = StringUtil.zeroLPad(date.getFullYear(), 4);
        var month = StringUtil.zeroLPad(date.getMonth() + 1, 2); //month is zero based
        var dayOfMonth = StringUtil.zeroLPad(date.getDate(), 2);

        return year + "-" + month + "-" + dayOfMonth + "T" + hour + ":" + min + ":" + secs;
    },
    formatTimeShortAmPm: function(date) {
        var hh = date.getHours();
        var ampm = (hh >= 12) ? "pm" : "am";
        hh = (hh > 12) ? hh-12 : hh;
        var hour   = hh;
        var min    = StringUtil.zeroLPad(date.getMinutes(), 2);
        return hour + ":" + min + " " + ampm;
    },
    formatTimeLong: function (date) {
        var hour   = StringUtil.zeroLPad(date.getHours(), 2);
        var min    = StringUtil.zeroLPad(date.getMinutes(), 2);
        var secs   = StringUtil.zeroLPad(date.getSeconds(), 2);
        var millis = StringUtil.zeroLPad(date.getMilliseconds(), 3);
        return hour + ":" + min + ":" + secs + "." + millis;
    },
    formatDateRecordings: function(date) {
        var hour   = StringUtil.zeroLPad(date.getHours(), 2);
        var min    = StringUtil.zeroLPad(date.getMinutes(), 2);

        var year = StringUtil.zeroLPad(date.getFullYear(), 4);
        var month = StringUtil.zeroLPad(date.getMonth() + 1, 2); //month is zero based
        var dayOfMonth = StringUtil.zeroLPad(date.getDate(), 2);

        return year + "-" + month + "-" + dayOfMonth + " " + hour + ":" + min;
    },
    formatDateTime: function (date) {
        var hour   = StringUtil.zeroLPad(date.getHours(), 2);
        var min    = StringUtil.zeroLPad(date.getMinutes(), 2);

        var year = StringUtil.zeroLPad(date.getFullYear(), 4);
        var month = StringUtil.zeroLPad(date.getMonth() + 1, 2); //month is zero based
        var dayOfMonth = StringUtil.zeroLPad(date.getDate(), 2);

        return dayOfMonth + "-" + month  + "-" + year + " " + hour + ":" + min;
    },
    parseFuzeDate: function(str) {
        if (!str) { return undefined; }

        var chunks = str.split(/T/g);
        var dateStr = chunks[0];
        var timeStr = chunks[1];

        var dateChunks = dateStr.split(/-/g);
        var timeChunks = timeStr.split(/:/g);

        var day = dateChunks[2];
        var month = parseInt(dateChunks[1]) - 1; //month is zero based
        var year = dateChunks[0];

        var hour = timeChunks[0];
        var min = timeChunks[1];
        var sec = timeChunks[2];

        var date = new Date();
        date.setUTCFullYear(year);
        date.setUTCMonth(month);
        date.setUTCDate(day);
        date.setUTCHours(hour);
        date.setUTCMinutes(min);
        date.setUTCSeconds(sec);

        return date;
    },
    getTime: function() {
        return Date.now();
    }
};

var XMLUtil = {
    elem: function(name) {
        return $('<' + name + '>');
    },
    parse: function(str) {
        return $(str);
    },
    serialize: function(jqNode) {
        return new XMLSerializer().serializeToString(jqNode.get(0));
    }
};

var StringUtil = {
    zeroLPad: function(n, len) {
        return new Array(Math.max( (len - String(n).length) + 1 , 0) ).join('0') + String(n);
    },
    ltrim: function(input) {
        if (!input) {
            return input;
        }
        return input.replace(/^\s+/, "");
    },
    rtrim: function(input) {
        if (!input) {
            return input;
        }
        return input.replace(/^\s+$/g, "");
    },
    trim: function(input) {
        if (!input) {
            return input;
        }

        return input.replace(/^\s+|\s+$/g, "");
    },
    trimLeadingSlash: function(input) {
        if (!input) {
            return input;
        }
        if (input.indexOf("/") === 0) {
            return input.substring(1);
        }
        return input;
    },
    formatMilisHuman: function(milis) {
        var ms = milis % 1000;
        var ss = parseInt(milis/1000) % 60;
        var mm = parseInt(milis/(60 * 1000)) % 60;
        var hh = parseInt(milis/(60 * 60 * 1000));
        var result = StringUtil.zeroLPad(hh, 2) + ":" + StringUtil.zeroLPad(mm, 2) + ":" + StringUtil.zeroLPad(ss, 2) + "." + StringUtil.zeroLPad(ms, 3);
        return result;
    },
    parseUrl: function(input) {
        if (typeof input !== "string") {
            return null;
        }
        var buff, protocol, host, port, path, query, hash;

        protocol = input.substring(0, input.indexOf(":"));
        buff = input.substring(input.indexOf(":")+3); // skip "://"
        host  = buff.substring(0, buff.indexOf("/"));
        path = buff.substring(buff.indexOf("/") + 1); // cut to the host:port part

        if (path.indexOf("?") !== -1) {

            query = path.substring(path.indexOf("?") + 1);

            path = path.substring(0, path.indexOf("?"));

            if (query.indexOf("#") !== -1) {
                hash  = query.substring(query.indexOf("#") + 1);
                query = query.substring(0, query.indexOf("#"));
            }
        }

        if (path.indexOf("#") !== -1) {
            hash = path.substring(path.indexOf("#") + 1);
            path = path.substring(0, path.indexOf("#"));
        }

        if (host.indexOf(":") !== -1) {
            port = host.substring(host.indexOf(":") + 1);
            host = host.substring(0, host.indexOf(":"));
        } else {
            port = (protocol.toLowerCase() === "http") ? 80 : 443;
        }
        var noQueryUrlString = protocol + "://" + host + ":" + port + "/" + path;
        return { protocol: protocol, host: host, port: port, path: path, query: query, hash: hash, urlString: input, noQueryUrlString: noQueryUrlString };
    },
    parseQuery: function(str) {
        var result = {};
        if (str) {
            var chunks = str.split(/&/g);
            for (var i=0; i<chunks.length; i++) {
                var kvPair = chunks[i].split(/=/g);
                var key = kvPair[0];
                var val = kvPair[1];
                result[key] = decodeURIComponent(val);
            }
        }
        return result;
    },
    addQueryParamsToUrl: function(urlString, params) {
        var queryString = StringUtil.toUrlQueryString(params);
        if (!queryString) {
            return urlString;
        }
        var hasQuery = urlString.indexOf("?") !== -1;
        var queryPrefix = (hasQuery) ? "&" : "?";
        var result = urlString + queryPrefix + queryString;
        return result;
    },
    toUrlQueryString: function(params) {
        var result = "";
        var hasParams = false;
        if (!params) {
            return result;
        }

        for (var i in params) {
            hasParams = true;
            result += window.encodeURIComponent(i);
            result += "=";
            result += window.encodeURIComponent(params[i]);
            result += "&";
        }
        if (!hasParams) {
            return result;
        }
        result = result.substring(0, result.length-1); //trim last &
        return result;
    },
    toInitials: function(str) {

        if (!str) return "";

        var chunks = str.split(" ");
        var initials = "";
        for (var i = 0; i < chunks.length; i++) {
            initials += chunks[i].charAt(0);
        }
        if (initials.length > 2) {
            initials = initials.substring(0,2);
        }
        return initials;
    },
    formatLineFeeds: function(str) {
        return str.replace(/\n/g, '<br/>');
    },
    linkify: function(str) {
        var result = new StringBuilder();

        var regex = /\bhttps?:\/\/[a-z0-9\.\-_](:\d+)?[^ \n\t<>()\[\]]*/gi;
        var match, lastEndIndex = 0;
        match = regex.exec(str);
        if (match === null) {
            return str;
        }

        while ( match !== null ) {
            var startIdx = match.index;
            var endIdx   = regex.lastIndex;
            var prevTextChunk = str.substring(lastEndIndex, startIdx);
            var url = str.substring(startIdx, endIdx);
            lastEndIndex = endIdx;
            match = regex.exec(str);

            //process
            result.append(prevTextChunk);
            result.append('<a href="').append(url).append('"')
                .append(' target="_blank">').append(url).append('</a>');

        }
        //append last text chunk
        if (lastEndIndex !== str.length) {
            result.append(str.substring(lastEndIndex));
        }

        return result.toString();
    },
    countBytesOrChars: function(str, bytesLimit) {
        var length = 0;
        for (var i = 0; i < str.length; i++) {
            var code = str.charCodeAt(i);
            if (code <= 0x7f) {
                length += 1;
            } else if (code <= 0x7ff) {
                length += 2;
            } else if (code >= 0xd800 && code <= 0xdfff) {
                // Surrogate pair: These take 4 bytes in UTF-8 and 2 chars in UCS-2
                // (Assume next char is the other [valid] half and just skip it)
                length += 4;
                i++;
            } else if (code < 0xffff) {
                length += 3;
            } else {
                length += 4;
            }

            if (bytesLimit && length >= bytesLimit) {
                return i + 1;
            }
        }

        if (bytesLimit && length >= bytesLimit) {
            return str.length;
        }

        //this works !
//        var l2 = encodeURI(str).split(/%..|./).length - 1;

        return length;
    },
    measureText: function(text, cssProps) {
        var clone = $('.text-measure-clone');
        var body = $('body');
        if (clone.length === 0) {
            clone = $('<div class="text-measure-clone"></div>').css({
                position: 'absolute',
                top: -10000,
                left: -10000,
                resize: 'none',
                wordWrap: 'break-word'
            }).appendTo(body);
        }
        clone.css(cssProps);
        /*
         {
         width: $self.width(),
         fontSize: $self.css('fontSize'),
         fontFamily: $self.css('fontFamily'),
         fontWeight: $self.css('fontWeight'),
         lineHeight: $self.css('lineHeight'),
         } */

        var val = StringUtil.escapeHtml(text);
        clone.html(val);
        var result = {width: clone.width(), height: clone.height()};
        clone.remove();
        return result;
    },
    escapeHtml: function(text) {
        var result = text
            .replace(/&/g, "&amp;")
            .replace(/</g, "&lt;")
            .replace(/>/g, "&gt;")
            .replace(/"/g, "&quot;")
            .replace(/'/g, "&#39;")
            .replace(/\//g, "&#x2F;");
        return result;
    },
    formatHtmlWhiteSpace: function(text) {
        var result = text
            .replace(/ /g, "&nbsp;")
            .replace(/\r\n/g, '<br/>')
            .replace(/\n/g, '<br/>');
        return result;
    },
    formatMeetingNumber: function(input) {
        if (!input || input.length === 0) {
            return input;
        }
        input = StringUtil.deleteAllNonNumericalCharacters(input);

        var formattedString = "";
        var meetingId = StringUtil.trim(input);
        for (var i=0; i < meetingId.length; i++) {
            // Format is 3-2-3 or 3-3-3. No legth check though, after 9 the format will be always 3-3-3-n

            if ((i > 0 && meetingId.length < 9  && (i === 3 || i === 5)) ||
                (i > 0 && meetingId.length >= 9  && (i % 3 === 0))) {
                formattedString += "-";
            }
            formattedString += meetingId.charAt(i);
        }

        return formattedString;
    },
    clearMeetingNumberFormat: function(input) {
        if (!input || input.length === 0)
        {
            return input;
        }

        return input.replace(/-/g, '');
    },
    deleteAllNonNumericalCharacters: function(input) {
        if (input.match(/\D/)) {
            return input.replace(/\D/g,'');
        }
        return input;
    },
    generateGUID: function() {
        var d = new Date().getTime();
        var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
            var r = (d + Math.random()*16)%16 | 0;
            d = Math.floor(d/16);
            return (c==='x' ? r : (r&0x7|0x8)).toString(16);
        });
        return uuid;
    },
    formatTimeCode: function(seconds) {
        var h = Math.floor(seconds/3600);
        var m = Math.floor((seconds%3600)/60);
        var s = Math.floor((seconds%3600)%60);

        var hfmt = StringUtil.zeroLPad(h, 2);
        var mfmt = StringUtil.zeroLPad(m, 2);
        var sfmt = StringUtil.zeroLPad(s, 2);

        if (h > 0) {
            return hfmt + ":" + mfmt + ":" + sfmt;
        }
        return mfmt + ":" + sfmt;
    }
};
var ArrayUtil = {};
ArrayUtil.remove = function(arr, value) {
    if (!arr) {
        return arr;
    }

    var result = [];
    for (var i=0; i<arr.length; i++) {
        if (arr[i] === value) {
            continue;
        }
        result.push(arr[i]);
    }
    return result;
};

var WebRTCUtil = {};
WebRTCUtil.cleanupMediaStream = function(stream) {
    if (!stream) {
        return;
    }
    var tracks, i, len;
    if (stream.getTracks) {
        tracks = stream.getTracks();
        for (i=0, len=tracks.length; i<len; i++) {
            tracks[i].stop();
        }
    } else {
        tracks = stream.getAudioTracks();
        for (i=0, len=tracks.length; i<len; i++) {
            tracks[i].stop();
        }
        tracks = stream.getVideoTracks();
        for (i=0, len=tracks.length; i<len; i++) {
            tracks[i].stop();
        }
    }
};

WebRTCUtil.muteAudioTrack = function(stream) {
    if (!stream) {
        return;
    }
    var tracks, i, len;
    tracks = stream.getAudioTracks();
    for (i=0, len=tracks.length; i<len; i++) {
        tracks[i].enabled = false;
    }
};

WebRTCUtil.unmuteAudioTrack = function(stream) {
    if (!stream) {
        return;
    }
    var tracks, i, len;
    tracks = stream.getAudioTracks();
    for (i=0, len=tracks.length; i<len; i++) {
        tracks[i].enabled = true;
    }
};

var Observable = function(asyncFlag) {

    var callbacks = [];

    asyncFlag = asyncFlag || false;

    this.size = function() {
        return callbacks.length;
    };

    this.attach = function(cb) {
        callbacks.push(cb);
    };

    this.detach = function(cb) {
        var result = [];
        for (var i=0; i < callbacks.length; i++) {
            if (callbacks[i] !== cb) {
                result.push(callbacks[i]);
            }
        }
        callbacks = result;
    };

    this.notify = function() {
        for (var i=0; i < callbacks.length; i++) {
            var callback = callbacks[i];
            if (asyncFlag) {
                (function(cb, args) {
                    setTimeout(function() {
                        cb.apply(cb, args);
                    }, 1);
                })(callback, arguments);
            } else {
                callback.apply(callback, arguments);
            }
        }
    };

    this.hasCallback = function(fn) {
        for (var i=0; i < callbacks.length; i++) {
            var callback = callbacks[i];
            if (callback === fn) {
                return true;
            }
        }
        return false;
    };

    this.clear = function () {
        callbacks = [];
    };
};

/**
 *
 * @param {type} store - js object where to store the observable instance
 * @param {type} key - the key to access the observable instance
 * @param {type} instance - this arg inside callbacks
 * @returns {function}
 *
 * it returns a function that acts as an entry point for subscribe and notify
 * depending on the arguments
 */
Observable.promiseLike = function(store, key, instance) {
    return function() {
        if (arguments.length === 1 && typeof arguments[0] === "function") {
            //subscribe
            if (!store[key]) {
                store[key] = new Observable();
            }
            store[key].attach(arguments[0]);
        } else {
            //async - to allow synchronious notify to work as well
            //ie: new CallProgress().started().started(function() {});
            //subscribers in the current call stack to be notified as well
            //when notify is called in the current call stack

            //notify
            var args = arguments;
            setTimeout(function() {
                if (store[key]) {
                    store[key].notify.apply(instance, args);
                }
            }, 1);
        }
        return instance;//return instnace to allow chaining
    };
};

var ObservableMap = function(asyncFlag) {

    var map = {};

    this.subscribe = function(name, cb) {
        if (typeof name !== "string") {
            throw new Error("subscribe() Invalid notification name " + name);
        }
        if (!map[name]) {
            map[name] = new Observable(asyncFlag);
        }
        map[name].attach(cb);
    };

    this.unsubscribe = function(name, cb) {
        if (typeof name !== "string") {
            throw new Error("unsubscribe() Invalid notification name " + name);
        }
        if (map[name]) {
            map[name].detach(cb);
        }
    };
    //args name, varargs arguments for subscribers
    this.notify = function() {
        if (arguments.length === 0) {
            return;
        }
        var name = arguments[0];

        if (typeof name !== "string") {
            throw new Error("notify() Invalid notification name " + name);
        }
        var restArgs = [];
        //skip first
        for (var i=1; i<arguments.length; i++) {
            restArgs.push(arguments[i]);
        }
        if (map[name]) {
            map[name].notify.apply(this, restArgs);
        }
    };

    this.clear = function() {
        map = {};
    };
};

var AsyncResult = function() {

    var store = {};
    var self = this;

    this.success = Observable.promiseLike(store,"success", this);
    this.error = Observable.promiseLike(store,"error", this);
    this.progress = Observable.promiseLike(store,"progress", this);

    this.clearSuccess = function(cb) {
        var prop = store['success'];
        if (prop) {
            prop.detach(cb);
        }
    };


    this.clearError = function(cb) {
        var prop = store['error'];
        if (prop) {
            prop.detach(cb);
        }
    };

    this.clearProgress = function(cb) {
        var prop = store['progress'];
        if (prop) {
            prop.detach(cb);
        }
    };

    this.pipe = function(ar) {
        if (!ar) {
            throw new Error("Undefined value passed to asyncResult.pipe");
        }
        if (ar === self) {
            throw new Error("Trying to pipe AsyncResult instance to itself");
        }
        self.progress(ar.progress);
        self.success(ar.success);
        self.error(ar.error);
    };
};

AsyncResult.create = function() {
    return new AsyncResult();
};

var Memo = {
    asyncResult: function(fn) {

        var cache = {};

        var argsHash = function(args) {
            return [].splice.call(args,0).join('\\n');
        };

        return function() {
            var result = AsyncResult.create();
            var hash = argsHash(arguments);
            //console.log(hash);
            var forceload = hash.indexOf('forceload') !== -1;
            var args = cache[hash];
            if (args !== undefined && !forceload) {
                //console.log("Data from cache");
                return result.success.apply(this, args);
            } else {
                if (args !== undefined && forceload) {
                    delete cache[hash];
                }
                //call original function and cache result arguments
                fn.apply(this, arguments).success(function() {
                    cache[hash] = arguments;
                    result.success.apply(this, arguments);
                }).error(result.error).progress(result.progress);
                return result;
            }
        };
    }
};

var DomUtil = {
    sumChildrenWidth: function(domElem, includeOuterWidth) {
        var childrenWidth = 0;
        domElem.children().each(function(index) {
            childrenWidth += $(this).outerWidth(includeOuterWidth);
        });
        return childrenWidth;
    },
    preventNonNumberCharactersInput: function(event, exceptions) {
        var chrTyped, chrCode = 0;
        if (event.charCode !== null) {
            chrCode = event.charCode;
        } else if (event.which !== null) {
            chrCode = event.which;
        } else if (event.keyCode !== null) {
            chrCode = event.keyCode;
        }

        if (chrCode === 0) {
            chrTyped = 'SPECIAL KEY';
        } else {
            chrTyped = String.fromCharCode(chrCode);
        }

        //Digits, special keys & backspace [\b] work as usual:
        if (chrTyped.match(/\d|[\b]|SPECIAL/)) {
            return true;
        }

        if (exceptions) {
            if (exceptions.allowSpace) {
                if (chrTyped.match(/ /)) {
                    return true;
                }
            }
        }

        var isMac = navigator.userAgent.indexOf(
                "Mac OS X") !== -1;
        if (event.altKey || event.ctrlKey || chrCode < 28 || (isMac && event.metaKey)) {
            return true;
        }

        //Any other input? Prevent the default response:
        if (event.preventDefault) {
            event.preventDefault();
        }

        event.returnValue = false;
        return false;
    }
};

var AsyncUtil = {};

/**
 *
 * @param {Array} arr
 * @param {function} end
 * @returns {undefined}
 *
 * Functions in the array are invoked sequentially
 * and data can be passed from function to function in the sequence
 * through calling next with the arguments that you want to pass
 *
 * @example
 * <pre>
 * AsyncUtil.waterfall([
 *     function(next) {
 *         //Do something and when finished call next.
 *         //Arguments are passed to the next function in the array.
 *         //In this case 1,2,3 are the values of a, b, c args
 *         next(1,2,3);
 *     },
 *     function(a, b, c, next) {
 *         //do something and call next
 *         next(4,5,6);
 *     }],
 *     function(c, d, e) {
 *         //do something when all the other functions are done
 *     }
 * );
 * </pre>
 */
AsyncUtil.waterfall = function(arr, end) {
    if ( !(arr instanceof Array) || arr.length === 0) {
        if (typeof end === "function") {
            end();
        }
        return;
    }

    var start = arr[0];
    var currentIndex = 0;
    var iter = function() {
        currentIndex++;
        if (currentIndex === arr.length) {
            if (typeof end === "function") {
                end.apply(this, arguments);
            }
        } else {
            //convert arguments to array
            var args = Array.prototype.slice.call(arguments);
            args.push(iter);
            arr[currentIndex].apply(this, args);
        }
    };
    start(iter);
};

/**
 *
 * @param {array} arr
 * @param {function} end
 * @returns {undefined}
 *
 * Invokes all the functions in paralel and call end when all are finished
 *
 */
AsyncUtil.parallel = function(arr, end) {
    if ( !(arr instanceof Array) || arr.length === 0) {
        if (typeof end === "function") {
            end();
        }
        return;
    }

    var count = arr.length;
    var currentCount = 0;
    var cb = function() {
        currentCount++;
        if (currentCount === count) {
            if (typeof end === "function") {
                end();
            }
        }
    };
    for (var i=0;i<arr.length; i++) {
        arr[i].call(this, cb);
    }
};